<?php
/**
* @copyright	Copyright (C) 2009 - 2012 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppDeltapay extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{	
		if($view != 'payment' || ($task != 'complete') ){
			return true;
		}
		
		$paymentKey = JRequest::getVar('Param1', null);
		if(!empty($paymentKey)){
			JRequest::setVar('payment_key', $paymentKey, 'POST');
			return true;
		}
		
		return true;
	}
	
	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
    	$guids = $this->_getGuids($payment);
    	
		//if first response is not proper then do nothing
    	if($guids['error_message']){
    		XiFactory::getApplication()->enqueueMessage($guids['error_message']);
    		return true;
    	}

    	$postUrl 		= "https://www.deltapay.gr/entry.asp";
    	$paymentKey   	= $payment->getKey();

    	$this->assign('guid1',$guids['guid1']);
    	$this->assign('guid2',$guids['guid2']);
    	$this->assign('postUrl',$postUrl);
	    $this->assign('paymentKey',$paymentKey);
	  	
	  	return $this->_render('form');
	}
	
	function  _getGuids($payment)
	{
		$invoice 	  = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
    	$user		  = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);	
		$paymentKey   = $payment->getKey();
		$email		  = $user->getEmail();
		$name		  = $user->getRealName();

		//Deltapay asks always for two demicals and have to use ',' as decimal seperator
		$amount       = number_format(round($invoice->getTotal(), '2'), '2', ',', '');
    	$merchantId   = $this->getAppParam('merchant_id');
    	$currencyCode = $invoice->getCurrency('isocode');
    	
    	if($currencyCode != 'EUR'){
    		$result['error_message'] = XiText::_('COM_PAYPLANS_PAYMENT_APP_DELTAPAY_CURRENCY_NOT_SUPPORTED');
    		return $result;
    	}
    		
    	//it supports on Euro for eshop transactions
    	$currencyCode = '978';
		
		$API_Endpoint = "https://www.deltapay.gr/getguid.asp";
		$nvpreq       = "MerchantCode=".$merchantId."&Charge=".$amount."&CurrencyCode=".$currencyCode."&CardHolderName=".$name."&CardHolderEmail=".$email."&Param1=".$paymentKey;
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $API_Endpoint);
		curl_setopt($ch, CURLOPT_VERBOSE, 1);
		
		// Turn off the server and peer verification (TrustManager Concept).
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_POST, 1);
		
		// Set the API operation, version, and API signature in the request.
		
		// Set the request as a POST FIELD for curl.
		curl_setopt($ch, CURLOPT_POSTFIELDS, $nvpreq);
		
		// Get response from the server.
		$httpResponse = curl_exec($ch);
		
		//create array of parameter
		$array_httpResponse = explode('<br>',$httpResponse);
		
		$result['guid1'] 			= isset($array_httpResponse[0])?$array_httpResponse[0]:'';
		$result['guid2'] 			= isset($array_httpResponse[1])?$array_httpResponse[1]:'';
		$result['error_message'] 	= isset($array_httpResponse[2])?$array_httpResponse[2]:'';
		
		return $result;
	}
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$txn_id       = isset($data['DeltaPayId'])?$data['DeltaPayId']:0;
		$subscr_id    =  0;
    	$parent_txn   =  0;
    	
		$invoice      = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
    	$transactions = $invoice->getTransactions();
    	
    	//if already have transaction for same DeltaPayId then don't create another transaction 
    	if(!empty($transactions)){
	    	foreach ($transactions as $transaction){
	    		if( $transaction->getAmount() != 0 && $transaction->getParam('DeltaPayId') == $txn_id){
	    			return true;
	    		}
	    	}
    	}
		
		// get the transaction instace of lib
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('gateway_txn_id', $txn_id)
					->set('gateway_parent_txn', $parent_txn)
					->set('gateway_subscr_id',$subscr_id)
					->set('params', PayplansHelperParam::arrayToIni($data));
					
		
		$func_name = isset($data['Result']) ? '_process_deltapay_result_'.JString::strtoupper($data['Result']) : 'EMPTY';
		
		if(method_exists($this, $func_name)){
			$result = $this->$func_name($data, $payment, $transaction, $invoice );
		}
		
		if(!$transaction->save()){
			$error['Payment ID']  = $payment->getId();
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data,'PayplansPaymentFormatter', '', true);
		}
					
		return true;
	}
	
	function _process_deltapay_result_1($data, $payment, $transaction, $invoice)
	{
		$amount  	  = str_replace(',','.',$data['Charge']);
		$transaction->set('amount',$amount)
		 			->set('message','COM_PAYPLANS_PAYMENT_APP_DELTAPAY_TRANSACTION_COMPLETED');
		 			
		return true;
	}
	
	function _process_deltapay_result_2($data, $payment, $transaction, $invoice)
	{
		$transaction->set('message','COM_PAYPLANS_PAYMENT_APP_DELTAPAY_TRANSACTION_NOT_COMPLETED');
		
		//create log
		$message 				  = $data['ErrorMessage'];
		$error['Payment ID'] 	  = $payment->getId();
		$error['DeltaPayId']	  = $data['DeltaPayId'];
		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
		
		return true;
	}
}
